import os
import json
import logging.config


def setup_logging(default_level=logging.INFO,
                  default_path="{}/logging.json".format(
                      os.getenv(
                          "LOG_DIR",
                          os.path.dirname(os.path.realpath(__file__)))),
                  env_key="LOG_CFG",
                  config_name=None):

    """
    Setup logging configuration
    """
    path = default_path
    file_name = default_path.split("/")[-1]
    if config_name:
        file_name = config_name

    path = "{}/{}".format(
                "/".join(default_path.split("/")[:-1]),
                file_name)

    value = os.getenv(env_key, None)
    if value:
        path = value
    if os.path.exists(path):
        with open(path, "rt") as f:
            config = json.load(f)
        logging.config.dictConfig(config)
        return

    else:
        cwd_path = os.getcwd() + "/network_pipeline/log/{}".format(
                                    file_name)
        if os.path.exists(cwd_path):
            with open(cwd_path, "rt") as f:
                config = json.load(f)
            logging.config.dictConfig(config)
            return
        rels_path = os.getcwd() + "/../log/{}".format(
                                    file_name)
        if os.path.exists(rels_path):
            with open(rels_path, "rt") as f:
                config = json.load(f)
            logging.config.dictConfig(config)
            return
        else:
            logging.basicConfig(level=default_level)
            return
# end of setup_logging


def build_logger(
    name=os.getenv(
        "LOG_NAME",
        "netp"),
    config="logging.json",
    log_level=logging.INFO,
    log_config_path="{}/logging.json".format(
        os.getenv(
            "LOG_CFG",
            os.path.dirname(os.path.realpath(__file__))))):
    """build_logger

    :param name: name that shows in the logger
    :param config: name of the config file
    :param log_level: level to log
    :param log_config_path: path to log config file
    """
    use_config = ("./log/{}").format(
                    "{}".format(
                        config))
    if not os.path.exists(use_config):
        use_config = log_config_path
        if not os.path.exists(use_config):
            use_config = ("./network_pipeline/log/{}").format(
                            "logging.json")
    # find the log processing

    setup_logging(
        default_level=log_level,
        default_path=use_config)

    return logging.getLogger(name)
# end of build_logger


def build_colorized_logger(
    name=os.getenv(
        "LOG_NAME",
        "netp"),
    config="colors-logging.json",
    log_level=logging.INFO,
    log_config_path="{}/logging.json".format(
        os.getenv(
            "LOG_CFG",
            os.path.dirname(os.path.realpath(__file__))))):
    """build_colorized_logger

    :param name: name that shows in the logger
    :param config: name of the config file
    :param log_level: level to log
    :param log_config_path: path to log config file
    """
    use_config = ("./log/{}").format(
                    "{}".format(
                        config))
    if not os.path.exists(use_config):
        use_config = log_config_path
        if not os.path.exists(use_config):
            use_config = ("./network_pipeline/log/{}").format(
                            "logging.json")
    # find the log processing

    setup_logging(
        default_level=log_level,
        default_path=use_config)

    return logging.getLogger(name)
# end of build_colorized_logger



import os
import struct
import json


SOURCE = os.getenv(
    "SOURCE_HOST",
    "localdev").strip().lstrip()
FORWARD_BROKER_URL = os.getenv(
    "FORWARD_BROKER_URL",
    "redis://localhost:6379/0").strip().lstrip()
FORWARD_SSL_OPTIONS = json.loads(os.getenv(
    "FORWARD_SSL_OPTIONS",
    "{}").strip().lstrip())
FORWARD_ENDPOINT_TYPE = os.getenv(
    "FORMAT_ET",
    "redis").strip().strip()
FORWARD_EXCHANGE = os.getenv(
    "FORWARD_EXCHANGE",
    "NEW_PACKETS").strip().lstrip()
FORWARD_ROUTING_KEY = os.getenv(
    "FORWARD_ROUTING_KEY",
    "NEW_PACKETS").strip().lstrip()
FORWARD_QUEUE = os.getenv(
    "FORWARD_QUEUE",
    "NEW_PACKETS").strip().lstrip()
DEBUG_PACKETS = bool(os.getenv(
    "DEBUG_PACKETS",
    "0").strip().lstrip() == "1")

# Prototype engine - for filtering off message contents
# not just src/dst ip and protocols
# not functional for all cases yet

INCLUDED_IGNORE_KEY = "CHANGE_TO_YOUR_OWN_KEY"

ETH_HEADER_FORMAT = "!6s6sH"
IP_HEADER_FORMAT = "!BBHHHBBH4s4s"
TCP_HEADER_FORMAT = "!HHLLBBHHH"
TCP_PSH_FORMAT = "!4s4sBBH"
UDP_HEADER_FORMAT = "!HHHH"
ICMP_HEADER_FORMAT = "!BBH"
ARP_HEADER_FORMAT = "2s2s1s1s2s6s4s6s4s"

SIZE_ETH_HEADER = struct.calcsize(ETH_HEADER_FORMAT)
SIZE_IP_HEADER = struct.calcsize(IP_HEADER_FORMAT)
SIZE_TCP_HEADER = struct.calcsize(TCP_HEADER_FORMAT)
SIZE_UDP_HEADER = struct.calcsize(UDP_HEADER_FORMAT)
SIZE_ICMP_HEADER = struct.calcsize(ICMP_HEADER_FORMAT)
SIZE_ARP_HEADER = struct.calcsize(ARP_HEADER_FORMAT)

VALID = 0
FILTERED = 1
INVALID = 2
ERROR = 3
UNSUPPORTED = 4
ETH_UNSUPPORTED = 5
IP_UNSUPPORTED = 6

UNKNOWN = 0
TCP = 1
UDP = 2
ICMP = 3
ARP = 4

ARP_PROTO_ETH = 9731
ICMP_PROTO_IP = 1
IP_PROTO_ETH = 8
TCP_PROTO_IP = 6
UDP_PROTO_IP = 17

IGNORED_REDIS_PORTS = [6379, 16379]
IGNORED_RABBITMQ_PORTS = [5672, 15672, 25672]



#!/usr/bin/env python

import os
import sys
import datetime
import time
import socket


def listen_on_udp_port():
    """listen_on_udp_port

    Run a simple server for processing messages over ``UDP``.

    ``UDP_LISTEN_ON_HOST`` - listen on this host ip address

    ``UDP_LISTEN_ON_PORT`` - listen on this ``UDP`` port

    ``UDP_LISTEN_SIZE`` - listen on to packets of this size

    ``UDP_LISTEN_SLEEP`` - sleep this number of seconds per loop

    ``UDP_LISTEN_SHUTDOWN_HOOK`` - shutdown if file is found on disk

    """

    host = os.getenv(
        "UDP_LISTEN_ON_HOST",
        "127.0.0.1").strip().lstrip()
    port = int(os.getenv(
        "UDP_LISTEN_ON_PORT",
        "17000").strip().lstrip())
    backlog = int(os.getenv(
        "UDP_LISTEN_BACKLOG",
        "5").strip().lstrip())
    size = int(os.getenv(
        "UDP_LISTEN_SIZE",
        "1024").strip().lstrip())
    sleep_in_seconds = float(os.getenv(
        "UDP_LISTEN_SLEEP",
        "0.5").strip().lstrip())
    shutdown_hook = os.getenv(
        "UDP_LISTEN_SHUTDOWN_HOOK",
        "/tmp/udp-shutdown-listen-server-{}-{}".format(
            host,
            port)).strip().lstrip()

    if os.path.exists(shutdown_hook):
        print(("Please remove the UDP shutdown hook file: "
               "\nrm -f {}")
              .format(shutdown_hook))
        sys.exit(1)

    now = datetime.datetime.now().isoformat()
    print(("{} - Starting UDP Server address={}:{} "
           "backlog={} size={} sleep={} shutdown={}")
          .format(
            now,
            host,
            port,
            backlog,
            size,
            sleep_in_seconds,
            shutdown_hook))

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.bind((host, port))

    msg = 0
    while 1:
        data = None
        address = None

        data, address = s.recvfrom(4096)

        if data:
            now = datetime.datetime.now().isoformat()
            print(("{} received UDP data={} ")
                  .format(
                    now,
                    data))
            msg += 1
            if msg > 1000000:
                msg = 0

            # if address:
            #     client.sendto("PROCESSED", address)
        else:
            time.sleep(sleep_in_seconds)

        if os.path.exists(shutdown_hook):
            now = datetime.datetime.now().isoformat()
            print(("{} detected shutdown "
                   "file={}")
                  .format(
                    now,
                    shutdown_hook))

    # end of loop

    print("Shutting down")

# end of listen_on_udp_port


if __name__ == '__main__':
    listen_on_udp_port()



import logging
import json
from collections import defaultdict
from network_pipeline.log.setup_logging import setup_logging
from network_pipeline.utils import ppj
from network_pipeline.consts import DEBUG_PACKETS


setup_logging()
name = "ptoj"
log = logging.getLogger(name)


def convert_pkt_to_json(pkg):
    """ convert_pkt_to_json
    Inspired by:
    https://gist.githubusercontent.com/cr0hn/1b0c2e672cd0721d3a07/raw/9144676ceb12dbd545e6dce366822bbedde8de2c/pkg_to_json.py
    This function convert a Scapy packet to JSON

    :param pkg: A scapy package
    :type pkg: objects

    :return: A JSON data
    :rtype: dict()

    """
    results = defaultdict(dict)

    try:
        for index in range(0, len(pkg)):

            layer = pkg[index]

            # Get layer name
            layer_tmp_name = str(layer.__dict__["aliastypes"][0])
            layer_start_pos = layer_tmp_name.rfind(".") + 1
            layer_name = layer_tmp_name[layer_start_pos:-2].lower()

            # Get the layer info
            tmp_t = {}
            for default_x, y in layer.__dict__["default_fields"].items():

                x = "default_{}".format(default_x)

                if DEBUG_PACKETS:
                    log.info("default: key={} val={}".format(x, y))

                try:
                    tmp_t["hex_default_{}".format(default_x)] = y.hex()
                except Exception:

                    # http://python3porting.com/differences.html#long
                    if y and not isinstance(y, (str,
                                                int,
                                                int,
                                                float,
                                                list,
                                                dict)):
                        if x in tmp_t:
                            tmp_t[x].update(convert_pkt_to_json(y))
                        else:
                            tmp_t[x] = y
                    else:
                        tmp_t[x] = y
            # end of fields

            results[layer_name] = tmp_t

            try:
                tmp_t = {}
                for fields_x, y in layer.__dict__["fields"].items():

                    if DEBUG_PACKETS:
                        log.info("fields: key={} val={}".format(x, y))

                    if fields_x == "qd":
                        if y:
                            tmp_t["fields_qd"] = json.loads(
                                    convert_pkt_to_json(y))
                    elif fields_x == "ar":
                        if y:
                            tmp_t["fields_ar"] = json.loads(
                                    convert_pkt_to_json(y))
                    elif fields_x == "an":
                        if y:
                            tmp_t["fields_an"] = json.loads(
                                    convert_pkt_to_json(y))
                    elif fields_x == "arcount":
                        if y:
                            tmp_t["fields_arcount"] = json.loads(
                                    convert_pkt_to_json(y))
                    elif fields_x == "ns":
                        if y:
                            """
                            'ns': <DNSRR  rrname='ubuntu.com.'
                            type=SOA rclass=IN ttl=1345
                            rdata=b'\x03ns1\tcanonical
                            \xc0\x19\nhostmaster\xc02xHl\x8e
                            \x00\x00*0\x00\x00\x0e\x10\x00
                            \t:\x80\x00\x00\x0e\x10' |>,
                            """
                            tmp_t["fields_ns"] = str(y)
                    elif fields_x == "proto":
                        if y:
                            tmp_t[x] = y
                    elif fields_x == "flags":
                        if y:
                            tmp_t[x] = y
                    elif fields_x == "ack":
                        if y:
                            tmp_t[x] = y
                    elif fields_x == "id":
                        if y:
                            tmp_t[x] = y
                    elif fields_x == "window":
                        if y:
                            tmp_t[x] = y
                    elif fields_x == "dataofs":
                        if y:
                            tmp_t[x] = y
                    elif fields_x == "frag":
                        if y:
                            tmp_t[x] = y
                    elif fields_x == "reserved":
                        if y:
                            tmp_t[x] = y
                    elif fields_x == "ttl":
                        if y:
                            tmp_t[x] = y
                    elif fields_x == "chksum":
                        if y:
                            tmp_t[x] = y
                    elif fields_x == "options":
                        if y:
                            cur_d = {}
                            try:
                                test = dict(y)
                                if "EOL" in test:
                                    cur_d["EOL"] = test["EOL"]
                                if "NOP" in test:
                                    cur_d["NOP"] = test["NOP"]
                                if "MSS" in test:
                                    cur_d["MSS"] = test["MSS"]
                                if "WScale" in test:
                                    cur_d["WScale"] = test["WScale"]
                                if "SAckOK" in test:
                                    cur_d["SAckOK"] = \
                                            test["SAckOK"].decode("utf-8")
                                if "SAck" in test:
                                    cur_d["SAck"] = test["SAck"]
                                if "Timestamp" in test:
                                    if test["Timestamp"]:
                                        cur_d["Timestamp"] = \
                                            test["Timestamp"][0]
                                if "AltChkSum" in test:
                                    cur_d["AltChkSum"] = test["AltChkSum"]
                                if "AltChkSumOpt" in test:
                                    cur_d["AltChkSumOpt"] = \
                                            test["AltChkSumOpt"]
                                if "Mood" in test:
                                    cur_d["Mood"] = test["Mood"]
                                if "Experiment" in test:
                                    cur_d["Experiment"] = test["Experiment"]
                            except Exception as exct:
                                log.error(("1 Failed parsing "
                                           "{}={} ex={}")
                                          .format(x,
                                                  y,
                                                  exct))
                                cur_d = str(y)
                            # end of parsing cur_d
                            tmp_t["fields_{}".format(fields_x)] = cur_d
                    elif fields_x == "urgptr":
                        if y:
                            cur_d = {}
                            try:
                                for f in y:
                                    cur_f = "{}_{}".format(fields_x,
                                                           f)
                                    try:
                                        cur_d[cur_f] = y.decode("utf-8")
                                    except Exception:
                                        cur_d["hex_" + cur_f] = y[f].hex()
                            except Exception as exct:
                                log.error(("2 Failed parsing "
                                           "{}={} ex={}")
                                          .format(x,
                                                  y,
                                                  exct))
                                cur_d = y
                            # end of parsing cur_d
                            tmp_t["fields_{}".format(fields_x)] = cur_d
                    else:
                        x = "{}".format(fields_x)

                        try:
                            hex_key = "hex_field_{}".format(fields_x)
                            if fields_x == "load":
                                try:
                                    tmp_t["load"] = y.decode("utf-8")
                                except Exception:
                                    tmp_t[hex_key] = y.hex()
                            else:
                                tmp_t[hex_key] = y.hex()
                        except Exception:
                            # http://python3porting.com/differences.html#long
                            if y and not isinstance(y, (str,
                                                        int,
                                                        int,
                                                        float,
                                                        list,
                                                        dict)):
                                if x in tmp_t:
                                    tmp_t[x].update(convert_pkt_to_json(y))
                                else:
                                    tmp_t[x] = y
                            else:
                                tmp_t[x] = y
                    # end of special handling:
                    # qd
                results[layer_name] = tmp_t

            except KeyError:
                # No custom fields
                pass

    except Exception:
        # Package finish -> do nothing
        pass

    if "padding" in results:
        try:
            if "load" in results["padding"]:
                results["padding"]["load"] = \
                    results["padding"]["load"].encode("utf-8").hex()
        except Exception:
            log.error(("failed parsing padding={}")
                      .format(results["padding"]))
    # end of fixing padding

    if "raw" in results:
        try:
            if "load" in results["raw"]:
                results["raw"]["load"] = \
                    results["raw"]["load"].encode("utf-8").hex()
        except Exception:
            log.error(("failed parsing raw={}")
                      .format(results["raw"]))
    # end of fixing raw

    if DEBUG_PACKETS:
        log.debug("")
        log.debug("pre json serialization:")
        log.debug(results)
        log.debug("post json.dumps:")
        log.debug(ppj(results))
        log.debug("")
    else:
        log.info(ppj(results))

    return results
# end of convert_pkt_to_json.py



import os
import sys
import logging
import json
import pandas as pd
from pandas.io.json import json_normalize
from celery_connectors.utils import ev
from network_pipeline.log.setup_logging import setup_logging
from network_pipeline.utils import ppj
from network_pipeline.utils import rnow
from network_pipeline.build_packet_key import build_packet_key
from antinex_client.consts import SUCCESS
from antinex_client.consts import FAILED
from antinex_client.consts import ERROR
from antinex_client.consts import LOGIN_FAILED
from antinex_client.consts import ANTINEX_PUBLISH_ENABLED
from antinex_client.consts import ANTINEX_PUBLISH_REQUEST_FILE
from antinex_client.consts import ANTINEX_USE_MODEL_NAME
from antinex_client.consts import ANTINEX_URL
from antinex_client.consts import ANTINEX_USER
from antinex_client.consts import ANTINEX_MISSING_VALUE
from antinex_client.build_ai_client_from_env import build_ai_client_from_env
from antinex_client.generate_ai_request import generate_ai_request


setup_logging()
name = "csv"
log = logging.getLogger(name)


class RecordPacketsToCSV:
    """RecordPacketsToCSV"""

    def __init__(self):
        """__init__"""

        self.recv_msgs = []

        # save every nth number of messages
        self.save_after_num = int(
            ev("SAVE_AFTER_NUM",
               "100"))

        # shutdown after this number of messages
        self.stop_after_num = int(
            ev("STOP_AFTER_NUM",
               "-1"))

        if self.save_after_num < 0:
            self.save_after_num = 1
        if self.stop_after_num < 0:
            self.stop_after_num = None

        # shutdown if this file is found
        self.stop_for_file = ev(
                "STOP_FILE",
                "/tmp/stop-recording-csv")

        self.dataset_name = ev(
                "DS_NAME",
                "netdata")

        self.save_dir = ev(
                "DS_DIR",
                "/tmp")

        self.save_to_file = ev(
                "OUTPUT_CSV",
                "{}/{}-{}.csv".format(
                    self.save_dir,
                    self.dataset_name,
                    rnow("%Y-%m-%d-%H-%M-%S")))

        self.archive_file = ev(
                "ARCHIVE_JSON",
                "{}/packets-{}-{}.json".format(
                    self.save_dir,
                    self.dataset_name,
                    rnow("%Y-%m-%d-%H-%M-%S")))

        self.debug = bool(ev(
                "DEBUG_PACKETS",
                "0") == "1")

        self.df = None
        self.last_df = None

        self.eth_keys = {"eth_id": "id"}
        self.ip_keys = {"ip_id": "id"}
        self.ipvsix_keys = {"ipvsix_id": "id"}
        self.icmp_keys = {"icmp_id": "id"}
        self.arp_keys = {"arp_id": "id"}
        self.tcp_keys = {"tcp_id": "id"}
        self.udp_keys = {"udp_id": "id"}
        self.dns_keys = {"dns_id": "id"}
        self.raw_keys = {"raw_id": "id"}
        self.pad_keys = {"pad_id": "id"}
        self.all_keys = {}
        self.all_keys_list = []

        self.all_eth = []
        self.all_ip = []
        self.all_ipvsix = []
        self.all_icmp = []
        self.all_arp = []
        self.all_tcp = []
        self.all_udp = []
        self.all_dns = []
        self.all_raw = []
        self.all_pad = []
        self.all_flat = []
        self.all_rows = []

        # noqa https://github.com/jay-johnson/antinex-client/blob/5fbcefaaed3d979b3c0829447b61592d5910ef22/antinex_client/build_ai_client_from_env.py#L19
        self.client = build_ai_client_from_env()

        # the client uses environment variables:
        # noqa https://github.com/jay-johnson/antinex-client/blob/5fbcefaaed3d979b3c0829447b61592d5910ef22/antinex_client/consts.py#L23
        # here is an example of what to export:
        # noqa https://github.com/jay-johnson/antinex-client/blob/master/examples/example-prediction.env
        self.request_dict = {}
        if ANTINEX_PUBLISH_ENABLED:
            with open(ANTINEX_PUBLISH_REQUEST_FILE, "r") as f:
                self.request_dict = json.loads(f.read())
        # if publishing is enabled

    # end of __init__

    def process_ether_frame(self,
                            id=None,
                            msg=None):
        """process_ether_frame

        Convert a complex nested json dictionary
        to a flattened dictionary and capture
        all unique keys for table construction

        :param id: key for this msg
        :param msg: ether frame for packet
        """

        # normalize into a dataframe
        df = json_normalize(msg)
        # convert to a flattened dictionary
        dt = json.loads(df.to_json())

        flat_msg = {}

        for k in dt:
            new_key = "eth_{}".format(k)
            flat_msg[new_key] = dt[k]["0"]
            if new_key not in self.eth_keys:
                self.eth_keys[new_key] = k
        # end of capturing all unique keys

        dt["eth_id"] = id
        self.all_eth.append(dt)

        log.debug("ETHER data updated:")
        log.debug(self.eth_keys)
        log.debug(self.all_eth)
        log.debug("")

        return flat_msg
    # end of process_ether_frame

    def process_ip_frame(self,
                         id=None,
                         msg=None):
        """process_ip_frame

        Convert a complex nested json dictionary
        to a flattened dictionary and capture
        all unique keys for table construction

        :param id: key for this msg
        :param msg: ip frame for packet
        """

        # normalize into a dataframe
        df = json_normalize(msg)
        # convert to a flattened dictionary
        dt = json.loads(df.to_json())

        flat_msg = {}

        for k in dt:
            new_key = "ip_{}".format(k)
            flat_msg[new_key] = dt[k]["0"]
            if new_key not in self.ip_keys:
                self.ip_keys[new_key] = k
        # end of capturing all unique keys

        dt["ip_id"] = id
        self.all_ip.append(dt)

        log.debug("IP data updated:")
        log.debug(self.ip_keys)
        log.debug(self.all_ip)
        log.debug("")

        return flat_msg
    # end of process_ip_frame

    def process_ipvsix_frame(self,
                             id=None,
                             msg=None):
        """process_ipvsix_frame

        Convert a complex nested json dictionary
        to a flattened dictionary and capture
        all unique keys for table construction

        :param id: key for this msg
        :param msg: ipv6 frame for packet
        """

        # normalize into a dataframe
        df = json_normalize(msg)
        # convert to a flattened dictionary
        dt = json.loads(df.to_json())

        flat_msg = {}

        for k in dt:
            new_key = "ipv6_{}".format(k)
            flat_msg[new_key] = dt[k]["0"]
            if new_key not in self.ipvsix_keys:
                self.ipvsix_keys[new_key] = k
        # end of capturing all unique keys

        dt["ipv6_id"] = id
        self.all_ipvsix.append(dt)

        log.debug("IPV6 data updated:")
        log.debug(self.ipvsix_keys)
        log.debug(self.all_ipvsix)
        log.debug("")

        return flat_msg
    # end of process_ip_frame

    def process_tcp_frame(self,
                          id=None,
                          msg=None):
        """process_tcp_frame

        Convert a complex nested json dictionary
        to a flattened dictionary and capture
        all unique keys for table construction

        :param id: key for this msg
        :param msg: tcp frame for packet
        """

        # normalize into a dataframe
        df = json_normalize(msg)
        # convert to a flattened dictionary
        dt = json.loads(df.to_json())

        flat_msg = {}

        for k in dt:
            new_key = "tcp_{}".format(k)
            flat_msg[new_key] = dt[k]["0"]
            if new_key not in self.tcp_keys:
                self.tcp_keys[new_key] = k
        # end of capturing all unique keys

        dt["tcp_id"] = id
        self.all_tcp.append(dt)

        log.debug("TCP data updated:")
        log.debug(self.tcp_keys)
        log.debug(self.all_tcp)
        log.debug("")

        return flat_msg
    # end of process_tcp_frame

    def process_udp_frame(self,
                          id=None,
                          msg=None):
        """process_udp_frame

        Convert a complex nested json dictionary
        to a flattened dictionary and capture
        all unique keys for table construction

        :param id: key for this msg
        :param msg: udp frame for packet
        """

        # normalize into a dataframe
        df = json_normalize(msg)
        # convert to a flattened dictionary
        dt = json.loads(df.to_json())

        flat_msg = {}

        for k in dt:
            new_key = "udp_{}".format(k)
            flat_msg[new_key] = dt[k]["0"]
            if new_key not in self.udp_keys:
                self.udp_keys[new_key] = k
        # end of capturing all unique keys

        dt["udp_id"] = id
        self.all_udp.append(dt)

        log.debug("UDP data updated:")
        log.debug(self.udp_keys)
        log.debug(self.all_udp)
        log.debug("")

        return flat_msg
    # end of process_udp_frame

    def process_dns_frame(self,
                          id=None,
                          msg=None):
        """process_dns_frame

        Convert a complex nested json dictionary
        to a flattened dictionary and capture
        all unique keys for table construction

        :param id: key for this msg
        :param msg: dns frame for packet
        """

        # normalize into a dataframe
        df = json_normalize(msg)
        # convert to a flattened dictionary
        dt = json.loads(df.to_json())

        flat_msg = {}

        for k in dt:
            new_key = "dns_{}".format(k)
            flat_msg[new_key] = dt[k]["0"]
            if new_key not in self.dns_keys:
                self.dns_keys[new_key] = k
        # end of capturing all unique keys

        dt["dns_id"] = id
        self.all_dns.append(dt)

        log.debug("DNS data updated:")
        log.debug(self.dns_keys)
        log.debug(self.all_dns)
        log.debug("")

        return flat_msg
    # end of process_dns_frame

    def process_icmp_frame(self,
                           id=None,
                           msg=None):
        """process_icmp_frame

        Convert a complex nested json dictionary
        to a flattened dictionary and capture
        all unique keys for table construction

        :param id: key for this msg
        :param msg: icmp frame for packet
        """

        # normalize into a dataframe
        df = json_normalize(msg)
        # convert to a flattened dictionary
        dt = json.loads(df.to_json())

        flat_msg = {}

        for k in dt:
            new_key = "icmp_{}".format(k)
            flat_msg[new_key] = dt[k]["0"]
            if new_key not in self.icmp_keys:
                self.icmp_keys[new_key] = k
        # end of capturing all unique keys

        dt["icmp_id"] = id
        self.all_icmp.append(dt)

        log.debug("ICMP data updated:")
        log.debug(self.icmp_keys)
        log.debug(self.all_icmp)
        log.debug("")

        return flat_msg
    # end of process_icmp_frame

    def process_arp_frame(self,
                          id=None,
                          msg=None):
        """process_arp_frame

        Convert a complex nested json dictionary
        to a flattened dictionary and capture
        all unique keys for table construction

        :param id: key for this msg
        :param msg: arp frame for packet
        """

        # normalize into a dataframe
        df = json_normalize(msg)
        # convert to a flattened dictionary
        dt = json.loads(df.to_json())

        flat_msg = {}

        for k in dt:
            new_key = "arp_{}".format(k)
            flat_msg[new_key] = dt[k]["0"]
            if new_key not in self.arp_keys:
                self.arp_keys[new_key] = k
        # end of capturing all unique keys

        dt["arp_id"] = id
        self.all_arp.append(dt)

        log.debug("ARP data updated:")
        log.debug(self.arp_keys)
        log.debug(self.all_arp)
        log.debug("")

        return flat_msg
    # end of process_arp_frame

    def process_raw_frame(self,
                          id=None,
                          msg=None):
        """process_raw_frame

        Convert a complex nested json dictionary
        to a flattened dictionary and capture
        all unique keys for table construction

        :param id: key for this msg
        :param msg: raw frame for packet
        """

        # normalize into a dataframe
        df = json_normalize(msg)
        # convert to a flattened dictionary
        dt = json.loads(df.to_json())

        flat_msg = {}

        for k in dt:
            new_key = "raw_{}".format(k)
            flat_msg[new_key] = dt[k]["0"]
            if new_key not in self.raw_keys:
                self.raw_keys[new_key] = k
        # end of capturing all unique keys

        dt["raw_id"] = id
        self.all_raw.append(dt)

        log.debug("RAW data updated:")
        log.debug(self.raw_keys)
        log.debug(self.all_raw)
        log.debug("")

        return flat_msg
    # end of process_raw_frame

    def process_pad_frame(self,
                          id=None,
                          msg=None):
        """process_pad_frame

        Convert a complex nested json dictionary
        to a flattened dictionary and capture
        all unique keys for table construction

        :param id: key for this msg
        :param msg: pad frame for packet
        """

        # normalize into a dataframe
        df = json_normalize(msg)
        # convert to a flattened dictionary
        dt = json.loads(df.to_json())

        flat_msg = {}

        for k in dt:
            new_key = "pad_{}".format(k)
            flat_msg[new_key] = dt[k]["0"]
            if new_key not in self.pad_keys:
                self.pad_keys[new_key] = k
        # end of capturing all unique keys

        dt["pad_id"] = id
        self.all_pad.append(dt)

        log.debug("PAD data updated:")
        log.debug(self.pad_keys)
        log.debug(self.all_pad)
        log.debug("")

        return flat_msg
    # end of process_pad_frame

    def build_flat_msg(self,
                       id=None,
                       msg=None):
        """build_flat_msg

        :param id: unique id for this message
        :param msg: message dictionary to flatten
        """

        flat_msg = {}

        if not id:
            log.error("Please pass in an id")
            return None
        if not msg:
            log.error("Please pass in a msg")
            return None

        for k in msg["data"]:
            if k == "ether":
                flat_msg.update(self.process_ether_frame(
                                    id=id,
                                    msg=msg["data"][k]))
            # end of ether
            elif k == "ip":
                flat_msg.update(self.process_ip_frame(
                                    id=id,
                                    msg=msg["data"][k]))
            # end of ip
            elif k == "ipv6":
                flat_msg.update(self.process_ipvsix_frame(
                                    id=id,
                                    msg=msg["data"][k]))
            # end of ipv6
            elif k == "tcp":
                flat_msg.update(self.process_tcp_frame(
                                    id=id,
                                    msg=msg["data"][k]))
            # end of tcp
            elif k == "udp":
                flat_msg.update(self.process_udp_frame(
                                    id=id,
                                    msg=msg["data"][k]))
            # end of udp
            elif k == "dns":
                flat_msg.update(self.process_dns_frame(
                                    id=id,
                                    msg=msg["data"][k]))
            # end of dns
            elif k == "icmp":
                flat_msg.update(self.process_icmp_frame(
                                    id=id,
                                    msg=msg["data"][k]))
            # end of icmp
            elif k == "arp":
                flat_msg.update(self.process_arp_frame(
                                    id=id,
                                    msg=msg["data"][k]))
            # end of arp
            elif k == "raw":
                flat_msg.update(self.process_raw_frame(
                                    id=id,
                                    msg=msg["data"][k]))
            # end of raw
            elif k == "padding":
                flat_msg.update(self.process_pad_frame(
                                    id=id,
                                    msg=msg["data"][k]))
            # end of pad
            else:
                log.error(("Unsupported frame type={} "
                           "please file an issue to track this "
                           "with data={} msg={}")
                          .format(k,
                                  ppj(msg["data"][k]),
                                  msg["data"]))
        # end of processing new message

        return flat_msg
    # end of build_flat_msg

    def build_all_keys_dict(self):
        """build_all_keys_dict"""

        log.info("finding keys")
        for k in self.eth_keys:
            ak = "{}".format(k)
            if ak not in self.all_keys:
                self.all_keys[ak] = k
        # end of building all eths
        for k in self.ip_keys:
            ak = "{}".format(k)
            if ak not in self.all_keys:
                self.all_keys[ak] = k
        # end of building all ips
        for k in self.ipvsix_keys:
            ak = "{}".format(k)
            if ak not in self.all_keys:
                self.all_keys[ak] = k
        # end of building all ipvsixs
        for k in self.icmp_keys:
            ak = "{}".format(k)
            if ak not in self.all_keys:
                self.all_keys[ak] = k
        # end of building all icmps
        for k in self.arp_keys:
            ak = "{}".format(k)
            if ak not in self.all_keys:
                self.all_keys[ak] = k
        # end of building all arps
        for k in self.tcp_keys:
            ak = "{}".format(k)
            if ak not in self.all_keys:
                self.all_keys[ak] = k
        # end of building all tcps
        for k in self.udp_keys:
            ak = "{}".format(k)
            if ak not in self.all_keys:
                self.all_keys[ak] = k
        # end of building all udps
        for k in self.dns_keys:
            ak = "{}".format(k)
            if ak not in self.all_keys:
                self.all_keys[ak] = k
        # end of building all dnss
        for k in self.raw_keys:
            ak = "{}".format(k)
            if ak not in self.all_keys:
                self.all_keys[ak] = k
        # end of building all raws
        for k in self.pad_keys:
            ak = "{}".format(k)
            if ak not in self.all_keys:
                self.all_keys[ak] = k
        # end of building all pads

        # this will be the columns for the csv
        for k in self.all_keys:
            self.all_keys_list.append(k)

        log.debug(("unique all_keys keys={} values={}")
                  .format(len(self.all_keys_list),
                          self.all_keys))

    # end of build_all_keys_dict

    def flatten_all(self):
        """flatten_all"""

        log.info("flattening - START")

        self.all_rows = []

        for idx, r in enumerate(self.all_flat):

            new_row = {"idx": idx}

            for k in self.all_keys_list:
                if k in r:
                    new_row[k] = r[k]
                else:
                    new_row[k] = None
            # end of for all keys

            self.all_rows.append(new_row)

        # end of all_keys

        log.info("flattening - END")
    # end of flatten_all

    def create_json_archive(self):
        """create_json_archive"""

        archive_data = {"packets": self.recv_msgs,
                        "dataset": self.dataset_name,
                        "num_packets": len(self.recv_msgs),
                        "created": rnow()}
        self.write_to_file(archive_data,
                           self.archive_file)

    # end of create_json_archive

    def convert_to_df(self):
        """convert_to_df"""

        log.info(("converting={}")
                 .format(len(self.all_rows)))

        if len(self.all_rows) == 0:
            return

        self.df = pd.DataFrame(self.all_rows).set_index("idx")

        if len(self.df) != len(self.all_rows):
            log.error(("Failed converting={} to rows={}")
                      .format(len(self.all_rows),
                              len(self.df)))
        else:
            log.info(("converted={} into rows={}")
                     .format(len(self.all_rows),
                             len(self.df)))

    # end of convert_to_df

    def write_to_file(self,
                      data_dict,
                      output_file_path):
        """write_to_file

        :param data_dict:
        :param output_file_path:
        """

        log.info("saving={}".format(output_file_path))
        with open(output_file_path, "w") as output_file:
            output_file.write(str(ppj(data_dict)))
    # end of write_to_file

    def save_df_as_csv(self):
        """save_df_as_csv"""

        if len(self.all_rows) == 0:
            log.info(("no df={} to save")
                     .format(self.df))
            return
        else:

            log.info(("saving "
                      "packets={} file={} rows={}")
                     .format(len(self.recv_msgs),
                             self.save_to_file,
                             len(self.df)))

            self.df.to_csv(self.save_to_file,
                           sep=",",
                           encoding="utf-8",
                           index=True)

            log.info(("done saving={}")
                     .format(self.save_to_file))
        # end of saving if the dataframe is there
    # end of save_df_as_csv

    def save_data(self):
        """save_data"""

        state = ""
        try:

            state = "create_json_archive"
            log.info("creating json archive")
            self.create_json_archive()

            state = "building_unique_keys"
            log.info("processing all unique keys")
            self.build_all_keys_dict()

            state = "flattening"
            log.info("flattening all data")
            self.flatten_all()

            state = "converting"
            log.info("converting to df")
            self.convert_to_df()

            state = "saving"
            log.info("saving to df")
            self.save_df_as_csv()

            if ANTINEX_PUBLISH_ENABLED:
                log.info(("publishing stream to rest={}")
                         .format(
                            ANTINEX_URL))
                self.publish_predictions_to_core()
            # end of if publishing to the core

        except Exception as e:
            log.error(("Failed state={} with ex={} to "
                       "save={}")
                      .format(state,
                              e,
                              self.save_to_file))
    # end of save_data

    def handle_msg(self,
                   body,
                   org_message):
        """handle_msg

        :param body: dictionary contents from the message body
        :param org_message: message object can ack, requeue or reject
        """

        if os.path.exists(self.stop_for_file):
            log.info(("Detected stop_file={} "
                      "shutting down")
                     .format(self.stop_for_file))

            # drop the message back in the queue
            # for next time
            org_message.requeue()

            sys.exit(1)
        # end of stop file detection

        try:

            log.debug(("handle body={}")
                      .format(ppj(body)))

            msg = body
            id = build_packet_key()
            recv_time = rnow()

            # this could be made into celery tasks...

            flat_msg = self.build_flat_msg(
                            id=id,
                            msg=msg)

            if not flat_msg:
                log.error(("Failed to build a flat message "
                           "for message={}")
                          .format(msg))
                return

            msg["id"] = id
            msg["received"] = recv_time
            if len(flat_msg) > 0:
                if self.debug:
                    log.info(ppj(flat_msg))
                flat_msg["id"] = id
                flat_msg["received"] = recv_time
                self.all_flat.append(flat_msg)
                self.recv_msgs.append(msg)
            # end of adding all flat messages

            already_saved = False
            num_recv = len(self.recv_msgs)
            if (num_recv % self.save_after_num) == 0:
                already_saved = False
                self.save_data()
            # end of saving a snapshot

            if self.stop_after_num:
                if num_recv >= self.stop_after_num:
                    if not already_saved:
                        self.save_data()
                    # avoid waiting on the save again
                    log.info("archive successful - purging buffer")
                    sys.exit(2)
                # shutdown - good for testing
            # if now set up for infinite consuming

        except Exception as e:
            log.error(("Failed processing msg={} "
                       "ex={}")
                      .format(body,
                              e))
        # end of processing message

        try:
            org_message.ack()
        except Exception as e:
            log.error(("Failed ack-ing msg={} "
                       "ex={}")
                      .format(body,
                              e))
        # end of acknowleding message was processed

        log.info("done handle")
    # end of handle_message

    def publish_predictions_to_core(self):
        """publish_predictions_to_core"""

        status = FAILED
        msg = "not started"

        try:

            msg = "generating request"
            log.info(msg)

            # noqa https://stackoverflow.com/questions/29815129/pandas-dataframe-to-list-of-dictionaries
            publish_req = generate_ai_request(
                predict_rows=self.df.fillna(
                    ANTINEX_MISSING_VALUE).to_dict("records"),
                req_dict=self.request_dict)

            if publish_req["status"] != SUCCESS:
                log.error(("failed generate_ai_request with err={}")
                          .format(
                            publish_req["error"]))
                status = ERROR

            else:

                msg = "publishing as user={} url={} model={}".format(
                    ANTINEX_USER,
                    ANTINEX_URL,
                    ANTINEX_USE_MODEL_NAME)

                log.info(msg)

                response = self.client.run_job(
                    body=publish_req["data"])

                if response["status"] == SUCCESS:
                    log.info("predictions sent")
                    status = SUCCESS
                elif response["status"] == FAILED:
                    log.error(("job failed with error='{}' with response={}")
                              .format(
                                response["error"],
                                response["data"]))
                    status = ERROR
                elif response["status"] == ERROR:
                    log.error(("job had an error='{}' with response={}")
                              .format(
                                response["error"],
                                response["data"]))
                    status = ERROR
                elif response["status"] == LOGIN_FAILED:
                    log.error(("job reported user was not able to log in "
                               "with an error='{}' with response={}")
                              .format(
                                response["error"],
                                response["data"]))
                    status = ERROR
                # logging for good/bad cases during publish
            # if generated a good request
        except Exception as e:
            log.error(("failed generating request last_step='{}' ex={}")
                      .format(
                        msg,
                        e))
        # end of try/ex

        return status
    # end of publish_predictions_to_core

# end of RecordPacketsToCSV



#!/usr/bin/env python

import os
import netifaces
import scapy.all as scapy


def send_arp_msg():
    """send_arp_msg

    Send an ``ARP`` message to the network device (``enp0s3`` by default).

    """

    dev = os.getenv(
            "ARP_INTERFACE",
            "enp0s3").strip().lstrip()
    network_details = netifaces.ifaddresses(
            dev)
    dst_ip = os.getenv(
            "ARP_DST_IP",
            network_details[2][0]["addr"])
    dst_mac = os.getenv(
            "ARP_DST_MAC",
            network_details[17][0]["addr"])

    print(("Sending ARP to mac={} ip={}")
          .format(dst_mac,
                dst_ip))

    answered, unanswered = scapy.srp(
                            scapy.Ether(
                                dst=dst_mac
                            ) / scapy.ARP(
                                pdst=dst_ip
                            ),
                            timeout=2,
                            verbose=False)

    if len(answered) > 0:
        print(answered[0][0].getlayer(
            scapy.ARP
        ).pdst + " is up")
    elif len(unanswered) > 0:
        print(unanswered[0].getlayer(
            scapy.ARP
        ).pdst + " is down")

# end of send_arp_msg


if __name__ == "__main__":
    send_arp_msg()



import logging
import uuid
import socket
import json
from struct import unpack
from network_pipeline.consts import ETH_HEADER_FORMAT
from network_pipeline.consts import IP_HEADER_FORMAT
from network_pipeline.consts import TCP_HEADER_FORMAT
from network_pipeline.consts import UDP_HEADER_FORMAT
from network_pipeline.consts import ICMP_HEADER_FORMAT
from network_pipeline.consts import ARP_HEADER_FORMAT
from network_pipeline.consts import SIZE_ETH_HEADER
from network_pipeline.consts import SIZE_IP_HEADER
from network_pipeline.consts import SIZE_TCP_HEADER
from network_pipeline.consts import SIZE_UDP_HEADER
from network_pipeline.consts import SIZE_ICMP_HEADER
from network_pipeline.consts import SIZE_ARP_HEADER
from network_pipeline.consts import VALID
from network_pipeline.consts import INVALID
from network_pipeline.consts import ERROR
from network_pipeline.consts import FILTERED
from network_pipeline.consts import UNKNOWN
from network_pipeline.consts import ETH_UNSUPPORTED
from network_pipeline.consts import IP_UNSUPPORTED
from network_pipeline.consts import TCP
from network_pipeline.consts import UDP
from network_pipeline.consts import ICMP
from network_pipeline.consts import ARP
from network_pipeline.consts import IP_PROTO_ETH
from network_pipeline.consts import ARP_PROTO_ETH
from network_pipeline.consts import TCP_PROTO_IP
from network_pipeline.consts import UDP_PROTO_IP
from network_pipeline.consts import ICMP_PROTO_IP
from network_pipeline.log.setup_logging import setup_logging

setup_logging()
name = "parser"
log = logging.getLogger(name)


# Get string of 6 characters as ethernet address into dash seperated hex string
def eth_addr(f):
    """eth_addr

    :param f: eth frame
    """
    data = "%.2x:%.2x:%.2x:%.2x:%.2x:%.2x" % (f[0],
                                              f[1],
                                              f[2],
                                              f[3],
                                              f[4],
                                              f[5])
    return data
# end of eth_addr


def unshift_flags(tcp_flags):
    '''
    De-shift the TCP flags to a string repr
    '''
    return (tcp_flags & 0x01,
            (tcp_flags >> 1) & 0x01,
            (tcp_flags >> 2) & 0x01,
            (tcp_flags >> 3) & 0x01,
            (tcp_flags >> 4) & 0x01,
            (tcp_flags >> 5) & 0x01,)
# end of unshift_flags


def build_key():
    """build_key"""
    return str(uuid.uuid4())
# end of build_key


def parse_network_data(data_packet=None,
                       include_filter_key=None,
                       filter_keys=[],
                       record_tcp=True,
                       record_udp=True,
                       record_arp=True,
                       record_icmp=True):
    """build_node

    :param data_packet: raw recvfrom data
    :param filter_keys: list of strings to filter
                        and remove baby-birding
                        packets to yourself
    :param record_tcp: want to record TCP frames?
    :param record_udp: want to record UDP frames?
    :param record_arp: want to record ARP frames?
    :param record_icmp: want to record ICMP frames?
    """

    node = {"id": build_key(),
            "data_type": UNKNOWN,
            "eth_protocol": None,
            "eth_src_mac": None,
            "eth_dst_mac": None,
            "eth_length": SIZE_ETH_HEADER,
            "ip_version_ih1": None,
            "ip_version": None,
            "ip_ih1": None,
            "ip_hdr_len": None,
            "ip_tos": None,
            "ip_tlen": None,
            "ip_id": None,
            "ip_frag_off": None,
            "ip_ttl": None,
            "ip_protocol": None,
            "ip_src_addr": None,
            "ip_dst_addr": None,
            "tcp_src_port": None,
            "tcp_dst_port": None,
            "tcp_sequence": None,
            "tcp_ack": None,
            "tcp_resrve": None,
            "tcp_data_offset": None,
            "tcp_flags": None,
            "tcp_adwind": None,
            "tcp_urg_ptr": None,
            "tcp_ffin": None,
            "tcp_fsyn": None,
            "tcp_frst": None,
            "tcp_fpsh": None,
            "tcp_fack": None,
            "tcp_furg": None,
            "tcp_header_size": None,
            "tcp_data_size": None,
            "tcp_data": None,
            "udp_header_size": None,
            "udp_data_size": None,
            "udp_src_port": None,
            "udp_dst_port": None,
            "udp_data_len": None,
            "udp_csum": None,
            "udp_data": None,
            "icmp_header_size": None,
            "icmp_data": None,
            "icmp_type": None,
            "icmp_code": None,
            "icmp_csum": None,
            "icmp_data_size": None,
            "arp_header_size": None,
            "arp_data": None,
            "arp_hw_type": None,
            "arp_proto_type": None,
            "arp_hw_size": None,
            "arp_proto_size": None,
            "arp_opcode": None,
            "arp_src_mac": None,
            "arp_src_ip": None,
            "arp_dst_mac": None,
            "arp_dst_ip": None,
            "arp_data_size": None,
            "target_data": None,
            "full_offset": None,
            "eth_header_size": None,
            "ip_header_size": None,
            "err": "",
            "stream": None,
            "filtered": None,
            "status": INVALID}

    err = "no_data"
    if not data_packet:
        node["error"] = err
        return node

    try:

        err = "missing_packet"
        packet = data_packet[0]

        if len(packet) < 21:
            node["status"] = INVALID
            node["error"] = "invalid packet={}".format(packet)
            return node

        err = "failed_parsing_ethernet"
        eth_packet_min = 0
        eth_packet_max = eth_packet_min + node["eth_length"]

        log.info(("unpacking ETH[{}:{}]")
                 .format(eth_packet_min,
                         eth_packet_max))

        eth_datagram = packet[eth_packet_min:eth_packet_max]
        eth_header = unpack(ETH_HEADER_FORMAT, eth_datagram)
        node["eth_protocol"] = socket.ntohs(eth_header[2])
        node["eth_src_mac"] = eth_addr(packet[0:6])
        node["eth_dst_mac"] = eth_addr(packet[6:12])
        log.debug(("eth src={} dst={} proto={}")
                  .format(node["eth_src_mac"],
                          node["eth_dst_mac"],
                          node["eth_protocol"]))

        node["eth_header_size"] = SIZE_ETH_HEADER

        # Is this an IP packet:
        if node["eth_protocol"] == IP_PROTO_ETH:

            ip_packet_min = SIZE_ETH_HEADER
            ip_packet_max = SIZE_ETH_HEADER + 20

            log.info(("unpacking IP[{}:{}]")
                     .format(ip_packet_min,
                             ip_packet_max))

            err = ("failed_parsing_IP[{}:{}]").format(
                    ip_packet_min,
                    ip_packet_max)

            # take the first 20 characters for the IP header
            ip_datagram = packet[ip_packet_min:ip_packet_max]

            ip_header = unpack(IP_HEADER_FORMAT, ip_datagram)
            # https://docs.python.org/2/library/struct.html#format-characters

            node["ip_header_size"] = SIZE_IP_HEADER

            node["ip_version_ih1"] = ip_header[0]
            node["ip_version"] = node["ip_version_ih1"] >> 4
            node["ip_ih1"] = node["ip_version_ih1"] & 0xF
            node["ip_hdr_len"] = node["ip_ih1"] * 4
            node["ip_tos"] = ip_header[1]
            node["ip_tlen"] = ip_header[2]
            node["ip_id"] = ip_header[3]
            node["ip_frag_off"] = ip_header[4]
            node["ip_ttl"] = ip_header[5]
            node["ip_protocol"] = ip_header[6]
            node["ip_src_addr"] = socket.inet_ntoa(ip_header[8])
            node["ip_dst_addr"] = socket.inet_ntoa(ip_header[9])

            log.debug("-------------------------------------------")
            log.debug("IP Header - Layer 3")
            log.debug("")
            log.debug(" - Version: {}".format(node["ip_version"]))
            log.debug(" - HDR Len: {}".format(node["ip_ih1"]))
            log.debug(" - TOS: {}".format(node["ip_tos"]))
            log.debug(" - ID: {}".format(node["ip_id"]))
            log.debug(" - Frag: {}".format(node["ip_frag_off"]))
            log.debug(" - TTL: {}".format(node["ip_ttl"]))
            log.debug(" - Proto: {}".format(node["ip_protocol"]))
            log.debug(" - Src IP: {}".format(node["ip_src_addr"]))
            log.debug(" - Dst IP: {}".format(node["ip_dst_addr"]))
            log.debug("-------------------------------------------")
            log.debug("")

            tcp_data = None
            udp_data = None
            arp_data = None
            icmp_data = None
            target_data = None

            eh = node["eth_header_size"]
            ih = node["ip_header_size"]

            log.debug(("parsing ip_protocol={} data")
                      .format(node["ip_protocol"]))

            if node["ip_protocol"] == TCP_PROTO_IP:

                packet_min = node["eth_length"] + node["ip_hdr_len"]
                packet_max = packet_min + 20

                # unpack the TCP packet
                log.info(("unpacking TCP[{}:{}]")
                         .format(packet_min,
                                 packet_max))

                err = ("failed_parsing_TCP[{}:{}]").format(
                        packet_min,
                        packet_max)

                tcp_datagram = packet[packet_min:packet_max]

                log.debug(("unpacking TCP Header={}")
                          .format(tcp_datagram))

                # unpack the TCP packet
                tcp_header = unpack(TCP_HEADER_FORMAT, tcp_datagram)

                node["tcp_src_port"] = tcp_header[0]
                node["tcp_dst_port"] = tcp_header[1]
                node["tcp_sequence"] = tcp_header[2]
                node["tcp_ack"] = tcp_header[3]
                node["tcp_resrve"] = tcp_header[4]
                node["tcp_data_offset"] = node["tcp_resrve"] >> 4

                node["tcp_flags"] = tcp_header[5]
                node["tcp_adwind"] = tcp_header[6]
                node["tcp_urg_ptr"] = tcp_header[7]

                # parse TCP flags
                flag_data = unshift_flags(node["tcp_flags"])
                node["tcp_ffin"] = flag_data[0]
                node["tcp_fsyn"] = flag_data[1]
                node["tcp_frst"] = flag_data[2]
                node["tcp_fpsh"] = flag_data[3]
                node["tcp_fack"] = flag_data[4]
                node["tcp_furg"] = flag_data[5]

                # process the TCP options if there are
                # currently just skip it
                node["tcp_header_size"] = SIZE_TCP_HEADER

                log.debug(("src={} dst={} seq={} ack={} doff={} flags={} "
                           "f urg={} fin={} syn={} rst={} "
                           "psh={} fack={} urg={}")
                          .format(node["tcp_src_port"],
                                  node["tcp_dst_port"],
                                  node["tcp_sequence"],
                                  node["tcp_ack"],
                                  node["tcp_data_offset"],
                                  node["tcp_flags"],
                                  node["tcp_urg_ptr"],
                                  node["tcp_ffin"],
                                  node["tcp_fsyn"],
                                  node["tcp_frst"],
                                  node["tcp_fpsh"],
                                  node["tcp_fack"],
                                  node["tcp_furg"]))
                # --------------------------------------------------------
                err = "failed_tcp_data"

                node["data_type"] = TCP
                node["tcp_header_size"] = (
                        node["ip_hdr_len"] + (node["tcp_data_offset"] * 4))
                node["tcp_data_size"] = len(packet) - node["tcp_header_size"]
                th = node["tcp_header_size"]
                node["full_offset"] = eh + ih + th
                log.info(("TCP Data size={} th1={} th2={} "
                          "offset={} value={}")
                         .format(node["tcp_data_size"],
                                 node["ip_hdr_len"],
                                 node["tcp_header_size"],
                                 node["full_offset"],
                                 tcp_data))
                err = "failed_tcp_data_offset"
                tcp_data = packet[node["full_offset"]:]
                target_data = tcp_data
                node["error"] = ""
                node["status"] = VALID
            elif node["ip_protocol"] == UDP_PROTO_IP:

                packet_min = node["eth_length"] + node["ip_hdr_len"]
                packet_max = packet_min + 8

                # unpack the UDP packet
                log.info(("unpacking UDP[{}:{}]")
                         .format(packet_min,
                                 packet_max))

                err = ("failed_parsing_UDP[{}:{}]").format(
                        packet_min,
                        packet_max)

                udp_datagram = packet[packet_min:packet_max]

                log.info(("unpacking UDP Header={}")
                         .format(udp_datagram))

                udp_header = unpack(UDP_HEADER_FORMAT, udp_datagram)
                node["udp_header_size"] = SIZE_UDP_HEADER

                node["udp_src_port"] = udp_header[0]
                node["udp_dst_port"] = udp_header[1]
                node["udp_data_len"] = udp_header[2]
                node["udp_csum"] = udp_header[3]

                node["data_type"] = UDP
                uh = node["udp_header_size"]
                node["full_offset"] = eh + ih + uh
                node["udp_data_size"] = len(packet) - node["udp_header_size"]
                log.info(("UDP Data size={} th1={} th2={} "
                          "offset={} value={}")
                         .format(node["udp_data_size"],
                                 node["ip_hdr_len"],
                                 node["udp_header_size"],
                                 node["full_offset"],
                                 udp_data))
                err = "failed_udp_data_offset"
                udp_data = packet[node["full_offset"]:]
                target_data = udp_data
                node["error"] = ""
                node["status"] = VALID
            elif node["ip_protocol"] == ICMP_PROTO_IP:

                # unpack the ICMP packet
                packet_min = node["eth_length"] + node["ip_hdr_len"]
                packet_max = packet_min + 4

                log.info(("unpacking ICMP[{}:{}]")
                         .format(packet_min,
                                 packet_max))

                err = ("failed_parsing_ICMP[{}:{}]").format(
                        packet_min,
                        packet_max)

                icmp_datagram = packet[packet_min:packet_max]

                log.info(("unpacking ICMP Header={}")
                         .format(icmp_datagram))

                icmp_header = unpack(ICMP_HEADER_FORMAT, icmp_datagram)

                node["icmp_header_size"] = SIZE_ICMP_HEADER

                node["icmp_type"] = icmp_header[0]
                node["icmp_code"] = icmp_header[1]
                node["icmp_csum"] = icmp_header[2]

                node["data_type"] = ICMP
                ah = node["icmp_header_size"]
                node["full_offset"] = eh + ih + ah
                node["icmp_data_size"] = len(packet) - node["icmp_header_size"]
                log.info(("ICMP Data size={} th1={} th2={} "
                          "offset={} value={}")
                         .format(node["icmp_data_size"],
                                 node["ip_hdr_len"],
                                 node["icmp_header_size"],
                                 node["full_offset"],
                                 icmp_data))
                err = "failed_icmp_data_offset"
                icmp_data = packet[node["full_offset"]:]
                target_data = icmp_data
                node["error"] = ""
                node["status"] = VALID
            else:
                node["error"] = ("unsupported_ip_protocol={}").format(
                                    node["ip_protocol"])
                node["status"] = IP_UNSUPPORTED
            # end of parsing supported protocols the final node data

            if node["status"] == VALID:

                log.debug("filtering")
                # filter out delimiters in the last 64 bytes
                if filter_keys:
                    err = "filtering={}".format(len(filter_keys))
                    log.debug(err)
                    for f in filter_keys:
                        if target_data:
                            if str(f) in str(target_data):
                                log.info(("FOUND filter={} "
                                         "in data={}")
                                         .format(f,
                                                 target_data))
                                node["error"] = "filtered"
                                node["status"] = FILTERED
                                node["filtered"] = f
                                break
                    # end of tagging packets to filter out of the
                    # network-pipe stream
                # if there are filters

                log.debug(("was filtered={}")
                          .format(node["filtered"]))

                if not node["filtered"]:
                    err = "building_stream"
                    log.debug(("building stream target={}")
                              .format(target_data))
                    stream_size = 0
                    if target_data:
                        try:
                            # convert to hex string
                            err = ("concerting target_data to "
                                   "hex string")
                            node["target_data"] = target_data.hex()
                        except Exception as e:
                            log.info(("failed converting={} to "
                                      "utf-8 ex={}")
                                     .format(target_data,
                                             e))
                            err = "str target_data"
                            node["target_data"] = target_data
                        # end of try/ex
                        stream_size += len(node["target_data"])
                    # end of target_data
                    log.debug(("serializing stream={}")
                              .format(node["target_data"]))
                    node_json = json.dumps(node)
                    data_stream = str("{} {}").format(node_json,
                                                      include_filter_key)
                    log.debug("compressing")

                    if stream_size:
                        node["stream"] = data_stream
                # end of building the stream

                log.debug("valid")
            else:
                log.error(("unsupported ip frame ip_protocol={}")
                          .format(node["ip_protocol"]))
            # end of supported IP packet protocol or not
        elif node["eth_protocol"] == ARP_PROTO_ETH:

            arp_packet_min = SIZE_ETH_HEADER
            arp_packet_max = SIZE_ETH_HEADER + 28

            log.info(("unpacking ARP[{}:{}]")
                     .format(arp_packet_min,
                             arp_packet_max))

            err = ("failed_parsing_ARP[{}:{}]").format(
                    arp_packet_min,
                    arp_packet_max)

            # take the first 28 characters for the ARP header
            arp_datagram = packet[arp_packet_min:arp_packet_max]

            arp_header = unpack(ARP_HEADER_FORMAT, arp_datagram)
            # https://docs.python.org/2/library/struct.html#format-characters

            node["arp_header_size"] = SIZE_ARP_HEADER
            node["arp_hw_type"] = arp_header[0].hex()
            node["arp_proto_type"] = arp_header[1].hex()
            node["arp_hw_size"] = arp_header[2].hex()
            node["arp_proto_size"] = arp_header[3].hex()
            node["arp_opcode"] = arp_header[4].hex()
            node["arp_src_mac"] = arp_header[5].hex()
            node["arp_src_ip"] = socket.inet_ntoa(arp_header[6])
            node["arp_dst_mac"] = arp_header[7].hex()
            node["arp_dst_ip"] = socket.inet_ntoa(arp_header[8])

            arp_data = ""
            node["arp_data"] = arp_data
            node["target_data"] = arp_data
            node["data_type"] = ARP
            node["status"] = VALID
            node["arp_data_size"] = len(packet) - node["arp_header_size"]
            node_json = json.dumps(node)
            data_stream = str("{} {}").format(node_json,
                                              include_filter_key)
            node["stream"] = data_stream
        else:
            node["error"] = ("unsupported eth_frame protocol={}").format(
                                node["eth_protocol"])
            node["status"] = ETH_UNSUPPORTED
            log.error(node["error"])
        # end of supported ETH packet or not

    except Exception as e:
        node["status"] = ERROR
        node["error"] = "err={} failed parsing frame ex={}".format(err,
                                                                   e)
        log.error(node["error"])
    # end of try/ex

    return node
# end of parse_network_data





def shutdown_consumers(num_workers=2,
                       tasks=None,
                       shutdown_msg="SHUTDOWN"):
    if tasks:
        # Add a poison pill for each consumer
        for i in range(num_workers):
            tasks.put(shutdown_msg)
# end of shutdown_consumers



#!/usr/bin/env python

import logging
import scapy.all as scapy
from celery_connectors.utils import ev
from network_pipeline.log.setup_logging import setup_logging
from network_pipeline.handle_packets import handle_packets


setup_logging()
# scapy capture agent
name = "cap-icmp"
log = logging.getLogger(name)


def capture_icmp_packets():
    """capture_icmp_packets

    Capture ``ICMP`` packets and call the ``handle_packets`` method

    Change the network interface by ``export CAP_DEVICE=eth0``

    """
    dev = ev(
        "CAP_DEVICE",
        "lo")

    """
    Ignore ports for forwarding to consolidators:

    Redis VM: 6379, 16379
    RabbitMQ VM: 5672, 15672, 25672

    """

    # http://biot.com/capstats/bpf.html
    default_filter = "icmp"
    custom_filter = ev(
        "NETWORK_FILTER",
        default_filter)

    log.info(("starting device={} filter={}")
             .format(
                dev,
                custom_filter))

    scapy.sniff(
        filter=custom_filter,
        prn=handle_packets)

    log.info("done")

# end of capture_icmp_packets


if __name__ == "__main__":
    capture_icmp_packets()
